@@ -4,6 +4,7 @@
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
 #undef HAVE_DEFOPEN
+#undef HAVE_FBCONSOLE
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
@@ -24,6 +25,7 @@
 #undef EXPANDED_BINDIR
 #undef EXPANDED_SBINDIR
 #undef EXPANDED_LIBEXECDIR
+#undef EXPANDED_LIBDIR
 #undef EXPANDED_GDMCONFIGDIR
 #undef EXPANDED_LOCALEDIR
 #undef EXPANDED_AUTHDIR
@@ -37,3 +39,8 @@
 #undef HAVE_LOGINCAP
 #undef HAVE_SYS_SOCKIO_H
 #undef HAVE_SCHED_YIELD
+#undef ENABLE_IPV6
+#undef HAVE_SELINUX
+#undef HAVE_PASSWDEXPIRED
+#undef HAVE_CHPASS
+#undef HAVE_LOGINRESTRICTIONS
